#!/usr/bin/env ruby
# Copyright (c) 2019, Aviz Networks. All rights reserved.
#
# This program and the accompanying materials are licensed and made available
# under the terms and conditions of the 3-clause BSD License that accompanies
# this distribution.
#
# The full text of the license may be found at
#
# https://opensource.org/licenses/BSD-3-Clause
#
# THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
# WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
require 'rest-client'
require 'json'
require_relative 'connect'
require_relative 'rest_utils'

# The System class provides a class implementation and methods for managing and setting up
# system configuration and images. This class presents an abstraction

class System
        @system_cfg  = '/api/devices'

        # This API gets the system meta data parameters
        #
        # Request URL: http://IP-ADDR:REST-PORT/api/devices
        #
        # @param conn [Class] Connect object to the node
        #
        # @return [RestClient::Request] Rest output from SONIC in JSON format as below
        #         {
        #          "mac": "b4:8c:db:b9:a0:00",
        #          "hwsku": "YYY",
        #          "hostname": "<string>",
        #          "platform": "x86_64",
        #          "version": <sonic-version>,
        #          "ASIC": <asic-type>
        #          "type": "LeafRouter"
        #          ...
        #         }
        def self.get_system_data(conn)
          url = form_url(conn, @system_cfg)
          hdr = form_hdr(conn)
          Rest.get(conn, url, hdr)
        end

        # This API updates system mac_addr and hostname
        #
        # Request URL: http://IP-ADDR:REST-PORT/api/devices
        #       payload format of key-value pairs
        #            {
        #             "mac_addr": "00:01:02:03:04:05",
        #             "hostname": "hostname"
        #            }
        #
        # @param conn [Class] Connect object to the node
        # @param mac_addr [String] Base MAC address of the system
        # @param hostname [String] Hostname of the system
        #
        # @return [RestClient::Request] Rest output from SONIC in JSON format
        def self.update_system_data(conn, mac_addr=nil, hostname=nil)
          url = form_url(conn, @system_cfg)
          hdr = form_hdr(conn)
          if mac_addr != nil
            params = {"mac_addr": mac_addr}
          end
          if hostname != nil
            p1 =  {"hostname": hostname}
            if mac_addr != nil
              params = params.merge(p1)
            else
              params = p1
            end
          end
          params = params.to_json
          Rest.put(conn, url, hdr, params)
        end

        # This API download and install system image
        #
        # Request URL: http://IP-ADDR:REST-PORT/api/devices
        #       payload format of key-value pairs
        #            {
        #             "image_url": "image_url"
        #            }
        #
        # @param conn [Class] Connect object to the node
        # @param image_url [String] URL to download system image from remote Web server (http ot https)
        #
        # @return [RestClient::Request] Rest output from SONIC in JSON format
        def self.download_system_img(conn, image_url)
          url = form_url(conn, @system_cfg)
          hdr = form_hdr(conn)
          params = {"image_url": image_url}
          params = params.to_json
          Rest.post(conn, url, hdr, params)
        end
end
